@@ -28,7 +28,7 @@ function(i, v, dim = NULL, dimnames = NULL)
     ## Note that this should never be true as it implies that either
     ## the class is wrong or the container is malformed.
     if (!.Call(R__valid_ssa, ssa))
-        stop("'ssa' not of class 'simple_sparse_array'")
+        stop("failed to create a valid 'simple_sparse_array' object")
     ssa
 }
 
@@ -94,15 +94,28 @@ dim.simple_sparse_array <-
 function(x)
     x$dim
 
+`dim<-.simple_sparse_array` <-
+function(x, value)
+{
+    value <- as.integer(value)
+    if(!length(value) || any(is.na(value)))
+        stop("invalid dim replacement value")
+    dx <- dim(x)
+    if(prod(value) != prod(dx))
+        stop("invalid dim replacement value")
+
+    x$i <- arrayInd(.Call(R_vector_index, x$dim, x$i), value)
+    x$dim <- value
+    x$dimnames <- NULL
+    
+    x
+}
+
 dimnames.simple_sparse_array <-
 function(x)
     x$dimnames
 
-
-## <TODO>
-## Add dim and dimnames setters.
-## </TODO>
-
+## FIXME we now have drop_simple_sparse_array
 `[.simple_sparse_array` <-
 function(x, ...)
 {
@@ -113,11 +126,21 @@ function(x, ...)
         return(x)
 
     nd <- length(x$dim)
+
+    ## Note there is a limit to representing integer numbers as 
+    ## doubles.
     spos <- function(i) {
+	if(!nrow(i)) 
+	    return(vector(mode = typeof(i), length = 0L))
         ## Scalar positions of array index matrices i in the usual row
         ## major ordering of arrays.
-        cpd <- cumprod(x$dim)
-        1L + row_sums((i - 1L) * rep(c(1L, cpd[-nd]), each = cpd[nd]))
+	if(ncol(i) > 1L) {
+	    ## This may not work on systems with BLAS issues
+	    ## as.vector(tcrossprod(c(1L, cumprod(x$dim[-nd])), i - 1L)) + 1L
+	    1L + row_sums((i - 1L) * rep(c(1L, cumprod(x$dim)[-nd]), 
+					 each = nrow(i)))
+	} else
+	    as.vector(i)
     }
 
     if(na == 2L) {
@@ -128,31 +151,68 @@ function(x, ...)
         else if(is.character(i))
             stop("Character subscripting currently not implemented.")
         else if(!is.matrix(i)) {
+	    ## 52-bit safe
+	    if(prod(x$dim) > 4503599627370496)
+	      stop("Numeric vector subscripting disabled for this object.")
+	     ## Shortcut
+	     if(!length(i)) 
+		return(vector(mode = typeof(x$v), length = 0L))
             ## Let's hope we have a vector.
             ## What if we have both negatives and positives?
-            if(all(i >= 0)) {
+	    if(is.double(i))
+		i <- trunc(i)
+            if(all(i >= 0, na.rm = TRUE)) {
                 i <- i[i > 0]
                 out <- vector(mode = typeof(x$v), length = length(i))
-                pos <- match(i, spos(x$i), 0L)
-                out[pos > 0L] <- x$v[pos]
-            } else if(all(i <= 0)) {
+		if(length(out)) {
+		    ## Missing values.
+		    is.na(i) <- i > prod(x$dim)
+		    is.na(out) <- is.na(i)
+		    i <- match(i, spos(x$i), 0L)
+		    out[i > 0L] <- x$v[i]
+		}
+            } else if(!any(is.na(i)) && all(i <= 0)) {
+		if(prod(x$dim) > 16777216L)
+		  stop("Negative vector subsripting disabled for this object.")
                 out <- vector(mode = typeof(x$v), prod(x$dim))
                 out[spos(x$i)] <- x$v
+		## NOTE this fails if NAs are introduced by 
+		##	coercion to integer. 
                 out <- out[i]
             }
             else stop("Cannot mix positive and negative subscripts.")
         }
         else {
+	     ## Shortcut
+	     if(!nrow(i)) 
+		return(vector(mode = typeof(x$v), length = 0L))
+	     ## Ignore dimensions. 
+	     if(ncol(i) != nd) 
+		return(do.call("[.simple_sparse_array", 
+			       list(x = x, as.vector(i))))
             ## Note that negative values are not allowed in a matrix
             ## subscript.
-            if((ncol(i) != nd) || (any(i < 0)))
+	    if(is.double(i))
+		i <- trunc(i)
+            if(any(i < 0, na.rm = TRUE))
                 stop("Invalid subscript.")
-            i <- i[!apply(i == 0, 1L, any), , drop = FALSE]
+	    k <- .Call(R_all_row, i > 0, FALSE)
+            i <- i[k, , drop = FALSE]
             out <- vector(mode = typeof(x$v), length = nrow(i))
-            ## This is not really the fastest way to match rows, but is
-            ## there an obvious better one?
-            pos <- match(split(i, row(i)), split(x$i, row(x$i)), 0L)
-            out[pos > 0L] <- x$v[pos]
+	    if(length(out)) {
+		if(any(i > rep(x$dim, each = nrow(i)), na.rm = TRUE))
+		    stop("subscript out of bounds")
+		## Missing values.
+		k <- k[k]
+		is.na(out) <- is.na(k)
+		rm(k)
+		## This is not really the fastest way to match rows, but is
+		## there an obvious better one?
+		## pos <- match(split(i, row(i)), split(x$i, row(x$i)), 0L)
+		storage.mode(i) <- "integer"
+		i <- .Call(R_match_matrix, x$i, i, 0L)[[2L]]
+		out[i > 0L] <- x$v[i]
+	    }
         }
     }
     else {
@@ -171,18 +231,29 @@ function(x, ...)
         ## Ready to go.
         dx <- x$dim
         pos <- rep.int(TRUE, length(x$v))
-        ind <- lapply(dx, seq_len)
+        ind <- vector("list", length = nd)
         for(k in seq_len(nd)) {
             i <- args[[k]]              # Given indices.
-            if(is.null(i)) next
+            if(is.null(i)) {
+		ind[[k]] <- seq_len(dx[k])
+		next
+	    }
             else if(!is.numeric(i))
                 stop("Only numeric multi-index subscripting is implemented.")
             else {
+		if (any(is.na(i)))
+		    stop("NA indices currently not allowed")
+		if(is.double(i))
+		    i <- trunc(i)
                 if(all(i >= 0)) {
                     i <- i[i > 0]
                     if(any(duplicated(i)))
                         stop("Repeated indices currently not allowed.")
+		    if(any(i > dx[k]))
+			stop("subscript out of bounds")
                 } else if(all(i <= 0))
+		    ## NOTE this fails if NAs are introduced by 
+		    ##	    coercion to integer. 
                     i <- seq_len(dx[k])[i]
                 else
                     stop("Cannot mix positive and negative subscripts.")
@@ -211,8 +282,8 @@ function(x, ...)
 print.simple_sparse_array <-
 function(x, ...)
 {
-    writeLines(sprintf("A simple sparse array of dimension %s.",
-                       paste(dim(x), collapse = "x")))
+    writeLines(gettextf("A simple sparse array of dimension %s.",
+                        paste(dim(x), collapse = "x")))
     invisible(x)
 }
 
@@ -242,12 +313,16 @@ function(a, perm = NULL, ...)
     
 }
 
+as.vector.simple_sparse_array <-
+function(x, mode = "any")
+    as.vector(as.array(x), mode)
+
 simple_sparse_zero_array <-
 function(dim, mode = "double")
 {
     ld <- length(dim)
     if (!ld)
-	stop("length-0 'dim' is invalid")
+	stop("'dim' must have positive length")
     simple_sparse_array(matrix(integer(), 0L, ld), vector(mode, 0L), dim)
 }
 
